// Progetto finale - Amedeo Di Biase
#include "gamelib.h"
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <ctype.h>
#include <time.h>

static void logo(void);

// Funzioni del "Imposta Gioco"
static void deallocazione(Mago*);                         //Line 192
static void creazione_personaggio(Mago*, int);            //Line 214
static void scelta_classe(Mago*);                         //Line 227
static int scelta_carte();                                //Line 243
static void crea_carta(Mago*, int);                       //Line 255
static char* check_tipo(Carta*);                          //Line 314 - Ritorna il tipo della carta passata come parametro
static char* check_classe(Mago*);                         //Line 324 - Ritorna la classe del mago passato come parametro
static void stampa_mazzi(Mago*, Mago*);                   //Line 333

// Funzioni del "Combatti"
static void gironi();                                     //Line 359
static char* duello(Mago*, Mago*);                        //Line 490
static void turno(Mago*, Mago*);                          //Line 531
static void stampa_mano(Mago*);                           //Line 613
static void stampa_campo(Mago*, Mago*);                   //Line 624
static void pesca(Mago*, unsigned short);                 //Line 643
static void inserisci_mano(Mago*, Carta*, unsigned short);//Line 665
static void gioca_carta(Mago*, Mago*);                    //Line 680
static void attacca(Mago*, Mago*);                        //Line 834

Mago Maghi[8];
int c = 0; // Variabile di controllo input
unsigned short Giocatori = 0;
unsigned short Impostato = 0; // Controlla se il gioco è stato già impostato
unsigned short Modalita = 0; // Duello o torneo
unsigned short Finite = 0; // Un giocatore ha finito le carte
unsigned short Num_Carte = 0;

void imposta_gioco() {
  time_t tempo; // Variabile su cui si basa la generazione casuale
  srand((unsigned)time(&tempo));
  int i = 0, j = 0;
  char stampa;

  if (Impostato == 1) {
    printf("\n\033[93mNotifica.\033[0m Il gioco è stato già impostato in precedenza, la vecchia configurazione è stata eliminata.\n");
    Giocatori = Impostato = Modalita = Finite = Num_Carte = 0;
    for (i = 0; i < 8; i++)
      deallocazione(&Maghi[i]);
  }

  printf("\nChe modalità vorresti giocare?\n");
  printf("> 1: Duello.\n");
  printf("> 2: Torneo.\n");
  printf("> 3: \033[1;31m\"Nuovo!\"\033[0m Battle Royale.\n\n");

  do {
    printf("\033[92mScelta:\033[0m ");
    scanf("%hu", &Modalita);
    while ((c = getchar()) != '\n' && c != EOF); // Pulizia buffer
    switch (Modalita) {
      case 1:
        creazione_personaggio(&Maghi[0], 0);
        creazione_personaggio(&Maghi[1], 1);
        Num_Carte = scelta_carte();

        for (i = 0; i < Num_Carte; i++) {
          crea_carta(&Maghi[0], i);
        }
        for (i = 0; i < Num_Carte; i++) {
          crea_carta(&Maghi[1], i);
        }

        printf("\nI mazzi sono stati generati.\n");
        stampa_mazzi(&Maghi[0], &Maghi[1]);
        break;

      case 2:
        printf("Il torneo può essere giocato da 4 o da 8 giocatori.\n");
        do {
          printf("\nQuanti giocatori ci sono? ");
          scanf("%hu", &Giocatori);
          while ((c = getchar()) != '\n' && c != EOF); // Pulizia buffer
          if (Giocatori != 4 && Giocatori != 8) {
            printf("Mi dispiace, ma le impostazioni prevedono solo \033[31m4 o 8 giocatori\033[0m.\n");
          }
        } while(Giocatori != 4 && Giocatori != 8);

        for (i = 0; i < Giocatori; i++) {
          creazione_personaggio(&Maghi[i], i);
        }

        Num_Carte = scelta_carte();

        for (i = 0; i < Giocatori; i++) {
          for (j = 0; j < Num_Carte; j++) {
            crea_carta(&Maghi[i], j);
          }
        }

        printf("\nI mazzi sono stati generati.\n");
        printf("Vuoi stampare i mazzi? (Il testo potrebbe occupare tanto spazio) (s/n) ");
        do {
          stampa = getchar();
          while ((c = getchar()) != '\n' && c != EOF); // Pulizia buffer
          if (stampa != 's' && stampa != 'n')
            printf("Inserisci \033[31ms\033[0m per si o \033[31mn\033[0m per no. ");
        } while (stampa != 's' && stampa != 'n');
        if (stampa == 's') {
          for (i = 0; i < Giocatori-1; i = i + 2) {
            stampa_mazzi(&Maghi[i], &Maghi[i+1]);
          }
        }
        break;

      case 3:
        printf("\nLa modalità Battle Royale è disponibile solo nella versione completa scaricabile da \033[1;3;4mhttps://tragic.altervista.org\033[0m\n\n");
        break;

      default:
        printf("\033[31mAttenzione!\033[0m Modalità non riconosciuta, per favore inserisci \033[31mun numero della lista\033[0m.\n");
    }
  } while (Modalita < 1 || Modalita > 2);

  Impostato = 1;
  printf("\nGioco impostato. Premere invio per continuare...");
  while (getchar() != '\n');
  logo();
  return;
}

void combatti() {
  if (Impostato == 0) {
    printf("\n\033[31mAttenzione!\033[0m Prima di giocare devi impostare il gioco!\n");
    return;
  }
  if (Modalita == 1) {
    duello(&Maghi[0], &Maghi[1]);
  } else {
    gironi();
  }

  while ((c = getchar()) != '\n' && c != EOF); // Pulizia buffer
  printf("\nIl gioco è finito. Premi invio per continuare...");
  while (getchar() != '\n');
  for(int i = 0; i < 8; i++)
    deallocazione(&Maghi[i]);
  Impostato = 0;
  Finite = 0;
  logo();
  return;
}

void termina_gioco() {
  for(int i = 0; i < 8; i++)
    deallocazione(&Maghi[i]);
  printf("\nGrazie per aver giocato a \033[1;36mTragic: L'accozzaglia\033[0m. Arrivederci Mago.\n\n");
  return;
}

static void logo() {
  system("clear");
  //Logo "Tragic: L'accozzaglia". Lo so che è un po' messy ¯\_(ツ)_/¯
  printf("\n");
  printf("\t\t\033[49m                                                       \033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m                                                                            \n");
  printf("\t\t\033[49m          \033[97m▄\033[107m  \033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m \033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m  \033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[107m                      \033[49m      \033[107m         \033[49m           \033[107m           \033[49m \033[97m▄\033[107m  \033[49m\033[97m▄\033[107m \033[49m\033[97m▄\033[107m   \033[49m        \033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m        \n");
  printf("\t\t\033[49m         \033[107m                                    \033[30m▄\033[49m   \033[107m     \033[49m     \033[107m\033[30m▄\033[107m      \033[49m        \033[107m      \033[49m       \033[97m▄\033[107m   \033[30m▄\033[49m       \033[107m   \033[49m    \033[107m     \033[49m     \033[97m▄\033[107m    \033[30m▄\033[107m\033[30m▄\033[49m \033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m     \033[49m        \n");
  printf("\t\t\033[49m         \033[107m     \033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[49m    \033[107m          \033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m    \033[49m    \033[107m    \033[49m      \033[97m▄\033[107m    \033[30m▄\033[49m       \033[97m▄\033[49m\033[97m▄\033[107m\033[30m▄\033[107m     \033[49m\033[97m▄\033[49m     \033[107m     \033[49m        \033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[49m     \033[107m    \033[49m    \033[97m▄\033[107m   \033[30m▄\033[49m       \033[107m\033[30m▄\033[107m   \033[30m▄\033[49m        \n");
  printf("\t\t\033[49m        \033[107m     \033[49m         \033[107m        \033[30m▄\033[49m          \033[107m    \033[49m    \033[107m    \033[49m     \033[97m▄\033[107m    \033[30m▄\033[49m      \033[97m▄\033[107m   \033[49m \033[107m\033[30m▄\033[107m     \033[49m     \033[107m    \033[30m▄\033[49m               \033[107m     \033[49m    \033[107m    \033[49m         \033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[49m         \n");
  printf("\t\t\033[49m        \033[107m    \033[49m           \033[107m       \033[49m           \033[107m    \033[49m    \033[107m     \033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[107m    \033[30m▄\033[49m        \033[107m   \033[30m▄\033[49m  \033[107m\033[30m▄\033[107m     \033[49m    \033[107m    \033[49m\033[97m▄\033[49m              \033[107m\033[30m▄\033[107m\033[30m▄\033[107m    \033[49m   \033[97m▄\033[107m    \033[49m                     \n");
  printf("\t\t\033[49m        \033[107m   \033[30m▄\033[49m          \033[97m▄\033[107m       \033[49m           \033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[49m     \033[107m     \033[30m▄\033[107m       \033[49m      \033[97m▄\033[107m    \033[49m    \033[107m      \033[49m   \033[107m     \033[49m      \033[107m      \033[49m   \033[107m     \033[49m   \033[107m\033[30m▄\033[107m     \033[49m                    \n");
  printf("\t\t\033[49m                      \033[107m        \033[49m                  \033[97m▄\033[107m     \033[49m   \033[107m\033[30m▄\033[107m     \033[49m\033[97m▄\033[49m   \033[97m▄\033[107m    \033[49m  \033[97m▄\033[107m  \033[49m \033[107m      \033[49m\033[97m▄\033[49m \033[107m      \033[49m\033[97m▄\033[49m     \033[107m     \033[49m   \033[107m     \033[49m    \033[107m     \033[49m\033[97m▄\033[49m\033[97m▄\033[49m                  \n");
  printf("\t\t\033[49m                      \033[107m\033[30m▄\033[107m       \033[49m                  \033[107m      \033[49m\033[97m▄\033[49m\033[97m▄\033[49m  \033[107m       \033[49m\033[97m▄\033[49m \033[107m         \033[49m   \033[107m      \033[49m\033[97m▄\033[49m \033[107m       \033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[107m     \033[49m   \033[107m     \033[49m\033[97m▄\033[49m\033[97m▄\033[49m  \033[107m\033[30m▄\033[107m       \033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m        \n");
  printf("\t\t\033[49m                      \033[107m        \033[49m                  \033[107m     \033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[49m   \033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[49m \033[107m\033[30m▄\033[107m  \033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[49m    \033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[49m  \033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m  \033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m   \033[49m   \033[107m  \033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[49m    \033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[49m        \n");
  printf("\t\t\033[49m                      \033[107m        \033[49m                  \033[107m  \033[49m                                                  \033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[49m   \033[107m\033[30m▄\033[49m                                 \n");
  printf("\t\t\033[49m                      \033[107m        \033[49m                 \033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m  \033[97m▄\033[49m\033[97m▄\033[49m                                                              \033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m   \033[97m▄\033[49m\033[97m▄\033[49m              \n");
  printf("\t\t\033[49m                      \033[107m        \033[49m                   \033[107m \033[49m    \033[107m  \033[49m                                                               \033[107m  \033[49m   \033[107m\033[30m▄\033[107m\033[30m▄\033[49m              \n");
  printf("\t\t\033[49m                      \033[97m▄\033[107m        \033[49m                  \033[107m \033[49m    \033[107m\033[30m▄\033[107m\033[30m▄\033[49m \033[107m  \033[30m▄\033[107m\033[30m▄\033[107m \033[49m\033[97m▄\033[49m  \033[107m  \033[30m▄\033[107m\033[30m▄\033[107m  \033[49m \033[107m  \033[30m▄\033[107m\033[30m▄\033[107m  \033[49m \033[107m   \033[30m▄\033[107m   \033[49m \033[107m  \033[30m▄\033[107m\033[30m▄\033[107m   \033[49m \033[107m  \033[30m▄\033[107m\033[30m▄\033[107m  \033[30m▄\033[49m \033[107m  \033[30m▄\033[107m\033[30m▄\033[107m \033[49m\033[97m▄\033[49m  \033[107m  \033[30m▄\033[107m\033[30m▄\033[107m  \033[49m  \033[107m  \033[49m  \033[107m\033[30m▄\033[107m \033[49m   \033[107m \033[30m▄\033[107m\033[30m▄\033[107m  \033[49m       \n");
  printf("\t\t\033[49m                      \033[107m        \033[30m▄\033[49m                  \033[107m \033[49m           \033[107m\033[30m▄\033[107m \033[49m \033[107m  \033[49m   \033[107m\033[30m▄\033[107m\033[30m▄\033[107m  \033[30m▄\033[49m  \033[107m\033[30m▄\033[107m\033[30m▄\033[107m  \033[30m▄\033[49m   \033[107m\033[30m▄\033[107m \033[49m \033[107m\033[30m▄\033[107m\033[30m▄\033[49m \033[97m▄\033[107m \033[30m▄\033[49m  \033[107m\033[30m▄\033[107m\033[30m▄\033[49m \033[107m  \033[49m       \033[107m  \033[49m  \033[107m \033[49m   \033[107m  \033[49m  \033[107m  \033[49m   \033[107m \033[49m       \033[107m \033[49m       \n");
  printf("\t\t\033[49m                     \033[107m          \033[49m                  \033[107m \033[49m    \033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[107m \033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m \033[49m \033[107m  \033[49m     \033[107m  \033[49m\033[97m▄\033[49m    \033[107m  \033[49m\033[97m▄\033[49m   \033[97m▄\033[107m \033[49m  \033[97m▄\033[107m  \033[49m \033[97m▄\033[49m\033[97m▄\033[49m  \033[107m  \033[30m▄\033[49m \033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[107m \033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m  \033[49m  \033[107m \033[30m▄\033[107m  \033[30m▄\033[49m   \033[107m  \033[49m   \033[107m \033[49m  \033[107m  \033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m \033[49m       \n");
  printf("\t\t\033[49m                     \033[107m\033[30m▄\033[107m         \033[49m\033[97m▄\033[49m               \033[97m▄\033[49m\033[97m▄\033[107m \033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[107m  \033[30m▄\033[107m \033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[107m  \033[49m\033[97m▄\033[49m \033[107m  \033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[107m \033[30m▄\033[107m  \033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[107m \033[30m▄\033[107m  \033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[107m  \033[49m \033[107m   \033[49m\033[97m▄\033[49m\033[97m▄\033[107m  \033[49m \033[107m   \033[49m\033[97m▄\033[49m\033[97m▄\033[107m  \033[30m▄\033[107m \033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[107m  \033[49m\033[97m▄\033[49m \033[107m     \033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[107m  \033[49m\033[97m▄\033[49m \033[97m▄\033[107m \033[49m\033[97m▄\033[49m\033[97m▄\033[107m \033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[107m  \033[49m\033[97m▄\033[49m\033[97m▄\033[49m     \n");
  printf("\t\t\033[49m                     \033[107m            \033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m\033[97m▄\033[49m                                                                         \033[107m \033[49m\033[97m▄\033[49m\033[97m▄\033[49m  \033[97m▄\033[107m  \033[49m                      \n");
  printf("\t\t\033[49m                     \033[107m                \033[49m                                                                          \033[107m\033[30m▄\033[107m   \033[30m▄\033[107m\033[30m▄\033[49m                       \n");
  printf("\t\t\033[49m                     \033[107m    \033[49m     \033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[49m  \033[107m\033[30m▄\033[107m\033[30m▄\033[49m                                                                                                       \n");
  printf("\t\t\033[49m                     \033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[107m\033[30m▄\033[49m                                                                                                                   \033[39m\n");
  printf("\n");
  return;
}

static void deallocazione(Mago* pMago) {
  int i = 0;
  Carta* pCarta = pMago -> Inizio_Mazzo;

  for (i = 0; i < 6; i++) {
    pMago -> Mano[i] = NULL;
    free(pMago -> Mano[i]);
  }
  for (i = 0; i < 4; i++) {
    pMago -> Campo[i] = NULL;
    free(pMago -> Campo[i]);
  }

  while (pCarta) {
    Carta* temp = pCarta -> Successiva;
    free(pCarta);
    pCarta = temp;
  }
  pMago -> PV = 20;
  pMago -> Inizio_Mazzo = NULL;
}

static void creazione_personaggio(Mago* pMago, int i) {
  pMago -> PV = 20;
  do {
    printf("\nInserisci il Nickname del %d° giocatore: ", i+1); // Chiedo il Nickname
    fgets(pMago -> Nickname, 10, stdin);
    pMago -> Nickname[strcspn(pMago -> Nickname, "\n")] = 0; // Rimuovo il newline finale per evitare problemi di stampa
    if (strcmp(pMago -> Nickname, "") == 0)
      printf("\n\033[31mAttenzione!\033[0m Il nome non può essere vuoto.");
  } while (strcmp(pMago -> Nickname, "") == 0);
  scelta_classe(pMago);
  pMago -> Inizio_Mazzo = NULL;
}

static void scelta_classe(Mago* pMago) {
  unsigned short Classe = 0;
  printf("\nScegli la classe del tuo mago.");
  printf("\n> \033[92m1\033[0m: \033[92mVita\033[0m: Le carte \033[92m\"Creatura\"\033[0m hanno il 50%% in più di statistiche.\n");
  printf("> \033[91m2\033[0m: \033[91mTenebre\033[0m: Le carte \"\033[091mInfliggi Danno\033[0m\" infliggono il doppio del danno.\n");
  printf("> \033[93m3\033[0m: \033[93mLuce\033[0m: Le carte \"\033[93mCura Danno\033[0m\" curano tre volte e mezzo il loro valore.\n");
  do {
    printf("\033[92mScelta:\033[0m ");
    scanf("%hu", &Classe);
    while ((c = getchar()) != '\n' && c != EOF); // Pulizia buffer
    if (Classe < 1 || Classe > 3)
      printf("\n\033[31mAttenzione!\033[0m Classe non riconosciuta, inserisci un numero tra 1 e 3.\n");
  } while (Classe < 1 || Classe > 3);
  pMago -> Classe = --Classe;
}

static int scelta_carte() {
  do {
    printf("\nInserire il numero di carte nel mazzo: ");
    scanf("%hu", &Num_Carte);
    while ((c = getchar()) != '\n' && c != EOF); // Pulizia buffer
    if (Num_Carte < 6 || Num_Carte > 30) {
      printf("\n\033[31mAttenzione!\033[0m Il numero di carte deve essere compreso tra 6 e 30.");
    }
  } while (Num_Carte < 6 || Num_Carte > 30);
  return Num_Carte;
}

static void crea_carta(Mago* pMago, int i) {
  char Nome[20][20] = {"Basilisco","Centauro","Cerbero","Colosso","Demone","Drago","Elfo","Fauno","Gargoyle","Gigante","Goblin","Ippogrifo","Kraken","Leviatano","Minotauro","Omuncolo","Orco","Pastore","Titano","Troll"};
  char Aggettivo[20][20] = {"Agguerrito","Bislungo","Buzzurro","Cafone","Filibustiere","Furioso","Gagliardo","Goffo","Incazzato","Lazzarone","Lestofante","Malandrino","Manigoldo","Mascalzone","Pauroso","Pazzo","Permaloso","Pusillanime","Sbilenco","Scalmanato",};

  Carta* New_Carta = (Carta*) (malloc(sizeof(Carta)));

  // Assegnazione Nome
  strcpy(New_Carta -> Nome, Nome[rand() % 20]);
  strcpy(New_Carta -> Aggettivo, Aggettivo[rand() % 20]);

  // Assegnazione Tipo
  short unsigned Probabilita_Tipo = rand() % 100;
  switch (Probabilita_Tipo) {
    case 0 ... 39:
      New_Carta -> Tipo = 0; // Creatura
      break;

    case 40 ... 64:
      New_Carta -> Tipo = 3; // Rimuovi Creatura
      break;

    case 65 ... 84:
      New_Carta -> Tipo = 1; // Infliggi Danno
      break;

    case 85 ... 99:
      New_Carta -> Tipo = 2; // Guarisci Danno
      break;
  }

  // Assegnazione Statistiche e Buff classe
  if (New_Carta -> Tipo != 3) {
    if (i <= 6)
      New_Carta -> Stats = 6 - i + rand() % 3 + 1;
    else
      New_Carta -> Stats = rand() % 3 + 1;
  } else {
    New_Carta -> Stats = 0;
  }
  if (New_Carta -> Tipo == 0 && pMago -> Classe == 0)      // Buff Vita
    New_Carta -> Stats = New_Carta -> Stats + (New_Carta -> Stats / 2);
  else if (New_Carta -> Tipo == 1 && pMago -> Classe == 1) // Buff Tenebre
    New_Carta -> Stats = New_Carta -> Stats * 2;
  else if (New_Carta -> Tipo == 2 && pMago -> Classe == 2) // Buff Luce
    New_Carta -> Stats = New_Carta -> Stats * 3.5;
  New_Carta -> Successiva = NULL;

  // Collegamento lista
  if (pMago -> Inizio_Mazzo == NULL) {
    pMago -> Inizio_Mazzo = New_Carta;
  } else {
    Carta* pCarta = pMago -> Inizio_Mazzo;
    while (pCarta -> Successiva != NULL)
      pCarta = pCarta -> Successiva;

    pCarta -> Successiva = New_Carta;
  }
}

static char* check_tipo(Carta* pCarta) {
  switch (pCarta -> Tipo) {
   case 0: return "Creatura";
   case 1: return "Infliggi Danno";
   case 2: return "Guarisci Danno";
   case 3: return "Rimuovi Creatura";
  }
  return 0;
}

static char* check_classe(Mago* pMago) {
  switch (pMago -> Classe) {
   case 0: return "Vita";
   case 1: return "Tenebre";
   case 2: return "Luce";
  }
  return 0;
}

static void stampa_mazzi(Mago* pPrimoMago, Mago* pSecondoMago) {
  if (pPrimoMago -> Inizio_Mazzo == NULL)
    printf("Non ci sono carte nel mazzo!");
  else {
    Carta* pCarta1 = pPrimoMago -> Inizio_Mazzo;
    Carta* pCarta2 = pSecondoMago -> Inizio_Mazzo;
    printf("\n\tMazzo di \033[31m%s\033[0m - Classe: %s.\t\t\t\t\t\t\t\tMazzo di \033[0;34m%s\033[0m - Classe: %s.\n\n", pPrimoMago -> Nickname, check_classe(pPrimoMago), pSecondoMago -> Nickname, check_classe(pSecondoMago));
    int Classe1 = pPrimoMago -> Classe;
    int Classe2 = pSecondoMago -> Classe;
    while (pCarta1 != NULL) {
      int Tipo1 = pCarta1 -> Tipo;
      int Tipo2 = pCarta2 -> Tipo;
      if (Classe1 == Tipo1 && Classe2 != Tipo2)
        printf("\033[94m%10s %-12s\033[0m - Tipo: \033[96m%-16s\033[0m - Stats: \033[95m%-2d\033[0m <- Potenziata \033[94m%20s %-12s\033[0m - Tipo: \033[96m%-16s\033[0m - Stats: \033[95m%-2d\033[0m             \n", pCarta1 -> Nome, pCarta1 -> Aggettivo, check_tipo(pCarta1), pCarta1 -> Stats, pCarta2 -> Nome, pCarta2 -> Aggettivo, check_tipo(pCarta2), pCarta2 -> Stats);
      else if (Classe1 != Tipo1 && Classe2 == Tipo2)
        printf("\033[94m%10s %-12s\033[0m - Tipo: \033[96m%-16s\033[0m - Stats: \033[95m%-2d\033[0m               \033[94m%20s %-12s\033[0m - Tipo: \033[96m%-16s\033[0m - Stats: \033[95m%-2d\033[0m <- Potenziata\n", pCarta1 -> Nome, pCarta1 -> Aggettivo, check_tipo(pCarta1), pCarta1 -> Stats, pCarta2 -> Nome, pCarta2 -> Aggettivo, check_tipo(pCarta2), pCarta2 -> Stats);
      else if (Classe1 == Tipo1 && Classe2 == Tipo2)
        printf("\033[94m%10s %-12s\033[0m - Tipo: \033[96m%-16s\033[0m - Stats: \033[95m%-2d\033[0m <- Potenziata \033[94m%20s %-12s\033[0m - Tipo: \033[96m%-16s\033[0m - Stats: \033[95m%-2d\033[0m <- Potenziata\n", pCarta1 -> Nome, pCarta1 -> Aggettivo, check_tipo(pCarta1), pCarta1 -> Stats, pCarta2 -> Nome, pCarta2 -> Aggettivo, check_tipo(pCarta2), pCarta2 -> Stats);
      else
        printf("\033[94m%10s %-12s\033[0m - Tipo: \033[96m%-16s\033[0m - Stats: \033[95m%-2d\033[0m               \033[94m%20s %-12s\033[0m - Tipo: \033[96m%-16s\033[0m - Stats: \033[95m%-2d\033[0m               \n", pCarta1 -> Nome, pCarta1 -> Aggettivo, check_tipo(pCarta1), pCarta1 -> Stats, pCarta2 -> Nome, pCarta2 -> Aggettivo, check_tipo(pCarta2), pCarta2 -> Stats);
      pCarta1 = pCarta1 -> Successiva;
      pCarta2 = pCarta2 -> Successiva;
    }
  }
}

static void gironi() {
  int i = 0, j = 0, k = 0;
  char SemiFina1[10], SemiFina2[10], SemiFina3[10], SemiFina4[10];
  char Finalista1[10], Finalista2[10], Vincitore[10];

  printf("\nChe il torneo abbia inizio!\n");
  sleep(1);

  if (Giocatori == 4) {
    strcpy(Finalista1, duello(&Maghi[0], &Maghi[1]));
    strcpy(Finalista2, duello(&Maghi[2], &Maghi[3]));
    printf("Passano in finale %s e %s\n", Finalista1, Finalista2);
    sleep(1);

    for (j = 0; j < 2; j++) {
      if (strcmp(Finalista1, Maghi[j].Nickname) == 0) {
        deallocazione(&Maghi[j]);
        for (i = 0; i < Num_Carte; i++)
          crea_carta(&Maghi[j], i);
        break;
      }
    }
    for (k = 2; k < 4; k++) {
      if (strcmp(Finalista2, Maghi[k].Nickname) == 0) {
        deallocazione(&Maghi[k]);
        for (i = 0; i < Num_Carte; i++)
          crea_carta(&Maghi[k], i);
        break;
      }
    }

    strcpy(Vincitore, duello(&Maghi[j], &Maghi[k]));
    printf("Il vincitore è %s", Vincitore);
    sleep(1);

    printf("\nGirone del torneo\n");
    printf(" %-9s──┐\n", Maghi[0].Nickname);
    printf("            ├┐\n");
    printf(" %-9s──┘├─ %-9s─┐\n", Maghi[1].Nickname, Finalista1);
    printf("             │            ├── %s\n", Vincitore);
    printf(" %-9s──┐├─ %-9s─┘\n", Maghi[2].Nickname, Finalista2);
    printf("            ├┘\n");
    printf(" %-9s──┘\n", Maghi[3].Nickname);

  } else {
    strcpy(SemiFina1, duello(&Maghi[0], &Maghi[1]));
    strcpy(SemiFina2, duello(&Maghi[2], &Maghi[3]));
    strcpy(SemiFina3, duello(&Maghi[4], &Maghi[5]));
    strcpy(SemiFina4, duello(&Maghi[6], &Maghi[7]));
    printf("Semifinali: %s VS %s, %s VS %s\n", SemiFina1, SemiFina2, SemiFina3, SemiFina4);
    sleep(1);

    for (j = 0; j < 2; j++) {
      if (strcmp(SemiFina1, Maghi[j].Nickname) == 0) {
        deallocazione(&Maghi[j]);
        for (i = 0; i < Num_Carte; i++)
          crea_carta(&Maghi[j], i);
        break;
      }
    }
    for (k = 2; k < 4; k++) {
      if (strcmp(SemiFina2, Maghi[k].Nickname) == 0) {
        deallocazione(&Maghi[k]);
        for (i = 0; i < Num_Carte; i++)
          crea_carta(&Maghi[k], i);
        break;
      }
    }

    strcpy(Finalista1, duello(&Maghi[j], &Maghi[k]));

    for (j = 4; j < 6; j++) {
      if (strcmp(SemiFina3, Maghi[j].Nickname) == 0) {
        deallocazione(&Maghi[j]);
        for (i = 0; i < Num_Carte; i++)
          crea_carta(&Maghi[j], i);
        break;
      }
    }
    for (k = 6; k < 8; k++) {
      if (strcmp(SemiFina4, Maghi[k].Nickname) == 0) {
        deallocazione(&Maghi[k]);
        for (i = 0; i < Num_Carte; i++)
          crea_carta(&Maghi[k], i);
        break;
      }
    }

    strcpy(Finalista2, duello(&Maghi[j], &Maghi[k]));

    for (j = 0; j < 4; j++) {
      if (strcmp(Finalista1, Maghi[j].Nickname) == 0) {
        deallocazione(&Maghi[j]);
        for (i = 0; i < Num_Carte; i++)
          crea_carta(&Maghi[j], i);
        break;
      }
    }
    for (k = 4; k < 8; k++) {
      if (strcmp(Finalista2, Maghi[k].Nickname) == 0) {
        deallocazione(&Maghi[k]);
        for (i = 0; i < Num_Carte; i++)
          crea_carta(&Maghi[k], i);
        break;
      }
    }
    printf("I finalisti sono %s e %s\n", Finalista1, Finalista2);
    sleep(1);
    strcpy(Vincitore, duello(&Maghi[j], &Maghi[k]));
    printf("Il vincitore è %s", Vincitore);
    sleep(1);

    printf("\nGirone del torneo\n");
    printf(" %-9s──┐\n", Maghi[0].Nickname);
    printf("            ├┐\n");
    printf(" %-9s──┘├ %-9s─┐\n", Maghi[1].Nickname, SemiFina1);
    printf("             │           ├──┐\n");
    printf(" %-9s──┐├ %-9s─┘  │\n", Maghi[2].Nickname, SemiFina2);
    printf("            ├┘              ├── %-9s┐\n", Finalista1);
    printf(" %-9s──┘               │            │\n", Maghi[3].Nickname);
    printf("                            │            ├── %s\n", Vincitore);
    printf(" %-9s──┐               │            │\n", Maghi[4].Nickname);
    printf("            ├┐              ├── %-9s┘\n", Finalista2);
    printf(" %-9s──┘├ %-9s─┐  │\n", Maghi[5].Nickname, SemiFina3);
    printf("             │           ├──┘\n");
    printf(" %-9s──┐├ %-9s─┘\n", Maghi[6].Nickname, SemiFina4);
    printf("            ├┘\n");
    printf(" %-9s──┘\n", Maghi[7].Nickname);
  }
}

static char* duello(Mago* pPrimoMago, Mago* pSecondoMago) {
  int i = 0, t = 0, primo = rand() % 2;

  for (i = 0; i < 5; i++)
    pesca(pPrimoMago, 1);
  for (i = 0; i < 5; i++)
    pesca(pSecondoMago, 1);

  printf("\n\033[1;35mÈ arivato il momento di combattere!\033[0m\n");

  while (pPrimoMago -> PV > 0 && pSecondoMago -> PV > 0 && !Finite) {
    printf("\tTurno \033[96m%d\033[0m\n", ++t);
    if (primo) {
      turno(pPrimoMago, pSecondoMago);
      turno(pSecondoMago, pPrimoMago);
    } else {
      turno(pSecondoMago, pPrimoMago);
      turno(pPrimoMago, pSecondoMago);
    }
  }
  if (pPrimoMago -> PV <= 0) {
    printf("\033[93m%s\033[94m è esausto, vince \033[93m%s\033[94m!\033[0m\n", pPrimoMago -> Nickname, pSecondoMago -> Nickname);
    return pSecondoMago -> Nickname;
  } else if (pSecondoMago -> PV <= 0) {
    printf("\033[93m%s\033[94m è esausto, vince \033[93m%s\033[94m!\033[0m\n", pSecondoMago -> Nickname, pPrimoMago -> Nickname);
    return pPrimoMago -> Nickname;
  } else if (pPrimoMago -> Inizio_Mazzo == NULL || pSecondoMago -> Inizio_Mazzo == NULL) {
    if (pPrimoMago -> PV > pSecondoMago -> PV) {
      printf("\033[94mUn mazzo è finito, vince \033[93m%s\033[94m, con \033[95m%d\033[94m Punti Vita!\033[0m", pPrimoMago -> Nickname, pPrimoMago -> PV);
      return pPrimoMago -> Nickname;
    } else if (pPrimoMago -> PV < pSecondoMago -> PV) {
      printf("\033[94mUn mazzo è finito, vince \033[93m%s\033[94m, con \033[95m%d\033[94m Punti Vita!\033[0m", pSecondoMago -> Nickname, pSecondoMago -> PV);
      return pSecondoMago -> Nickname;
    } else if (pPrimoMago -> PV == pSecondoMago -> PV) {
      printf("\033[94mUn mazzo è finito ed entrambi i giocatori hanno la stessa vita, è un pareggio!\033[0m");
      return 0;
    }
  }
  return 0;
}

static void turno(Mago* pPrimoMago, Mago* pSecondoMago) {
  unsigned short opzione = 0, pescato = 0, giocato = 0, attaccato = 0;
  int i = 0;

  do {
    if (pPrimoMago -> PV > 0 && pSecondoMago -> PV > 0 && !Finite) {
      if (opzione == 0) { // Stampo il messaggio solo se il turno è appena iniziato
        printf("È il turno di \033[93m%s\033[0m\n", pPrimoMago -> Nickname);
        sleep(1);
      }
      printf("\n\033[93mCosa vuoi fare?\033[0m\n");
      printf("> 1: Stampa la mia mano.\n");
      printf("> 2: Stampa le carte sul campo.\n");
      printf("> 3: Pesca una carta.\n");
      printf("> 4: Gioca una carta.\n");
      printf("> 5: Attacca.\n");
      printf("> 6: Passa il turno.\n");
      printf("\033[92mScelta:\033[0m ");
      scanf("%hu", &opzione);
      while ((c = getchar()) != '\n' && c != EOF); // Pulizia buffer

      switch (opzione) {
        case 1:
          stampa_mano(pPrimoMago);
          break;

        case 2:
          stampa_campo(pPrimoMago, pSecondoMago);
          break;

        case 3:
          if (pescato == 0)
            pesca(pPrimoMago, 0);
          else
            printf("\nHai già pescato in questo turno.\n");
          pescato = 1;
          break;

        case 4:
          if (giocato == 0) {
            i = 0;
            while (pPrimoMago -> Mano[i] == NULL && i < 6) {
              i++;
            }
            if (i < 6) {
              gioca_carta(pPrimoMago, pSecondoMago);
              giocato = 1;
            } else
              printf("\nNon hai carte in mano, non puoi giocare.\n");
          } else
            printf("\nHai già giocato una carta in questo turno.\n");
          break;

        case 5:
          if (attaccato == 0) {
            i = 0;
            while (pPrimoMago -> Campo[i] == NULL && i < 4) {
              i++;
            }
            if (i < 4) {
              attacca(pPrimoMago, pSecondoMago);
              attaccato = 1;
            } else
              printf("\nNon hai carte schierate sul campo, non puoi attaccare.\n");
          } else
            printf("\nHai già attacato in questo turno.\n");
          break;

        case 6:
          printf("\n");
          break;

        default:
          printf("\n\033[31mAttenzione!\033[0m Scelta non valida, per favore \033[31mscegli un numero dalla lista\033[0m.\n");
      }
    } else
      break;
  } while (opzione != 6);
  system("clear");
  return;
}

static void stampa_mano(Mago* pMago) {
  printf("\nI tuoi Punti Vita: \033[95m%d\033[0m\n", pMago -> PV);
  for (int i = 0; i < 6; i++) {
    if (pMago -> Mano[i] != NULL) {
      Carta* pCarta = pMago -> Mano[i];
      printf ("\033[93m%d° Slot mano\033[0m: \033[94m%10s %-12s\033[0m - Tipo: \033[96m%-16s\033[0m - Stats: \033[95m%-2d\033[0m\n", i+1, pCarta -> Nome, pCarta -> Aggettivo, check_tipo(pCarta), pCarta -> Stats);
    } else
      printf("\033[93m%d° Slot mano\033[0m: Vuoto\n", i+1);
  }
}

static void stampa_campo(Mago* pPrimoMago, Mago* pSecondoMago) {
  printf("\n");
  for (int i = 0; i < 4; i++) {
    if (pPrimoMago -> Campo[i] != NULL) {
      Carta* pCarta1 = pPrimoMago -> Campo[i];
      printf("\033[94m%d° Slot campo alleato\033[0m: \033[96m%10s %-12s\033[0m - Stats: \033[95m%-2d\033[0m\n", i+1, pCarta1 -> Nome, pCarta1 -> Aggettivo, pCarta1 -> Stats);
    } else
      printf("\033[94m%d° Slot campo alleato\033[0m: Vuoto\n", i+1);
  }
  printf("\n");
  for (int i = 0; i < 4; i++) {
    if (pSecondoMago -> Campo[i] != NULL) {
      Carta* pCarta1 = pSecondoMago -> Campo[i];
      printf("\033[91m%d° Slot campo avversario\033[0m: \033[96m%10s %-12s\033[0m - Stats: \033[95m%-2d\033[0m\n", i+1, pCarta1 -> Nome, pCarta1 -> Aggettivo, pCarta1 -> Stats);
    } else
      printf("\033[91m%d° Slot campo avversario\033[0m: Vuoto\n", i+1);
  }
}

static void pesca(Mago* pMago, unsigned short Prima_Pesca) {
  Carta* pCarta = pMago -> Inizio_Mazzo;
  if (pCarta == NULL) {
    printf("\nHai finito le carte, la vittoria va al giocatore con più HP.\n");
    while (getchar() != '\n');
    Finite = 1;
    return;
  } else {
    if (pCarta -> Successiva == NULL) {
      printf("\nUltima carta nel mazzo.");
      inserisci_mano(pMago, pCarta, Prima_Pesca);
      pMago -> Inizio_Mazzo = NULL;
    } else {
      while ((pCarta -> Successiva) -> Successiva != NULL)
        pCarta = pCarta -> Successiva;
      inserisci_mano(pMago, pCarta -> Successiva, Prima_Pesca);
      pCarta -> Successiva = NULL;
      free(pCarta -> Successiva);
    }
  }
}

static void inserisci_mano(Mago* pMago, Carta* Carta_Pescata, unsigned short Prima_Pesca) {
  int i = 0;
  for (i = 0; i < 6; i++) {
    if (pMago -> Mano[i] == NULL) {
      pMago -> Mano[i] = Carta_Pescata;
      if (!Prima_Pesca)
        printf("\nHai pescato \033[94m%s %s\033[0m - Tipo: \033[96m%s\033[0m - Stats: \033[95m%d\033[0m\n", Carta_Pescata -> Nome, Carta_Pescata -> Aggettivo, check_tipo(Carta_Pescata), Carta_Pescata -> Stats);
      break;
    }
  }
  if (i == 6) {
    printf("\n\033[31mAttenzione!\033[0m La tua Mano è piena, la carta scelta verrà distrutta.\n");
  }
}

static void gioca_carta(Mago* pPrimoMago, Mago* pSecondoMago) {
  unsigned short carta = 0, Bersaglio = 0, i = 0, j = 0, k = 0, l = 0;
  do {
    stampa_mano(pPrimoMago);
    do {
      printf("\nQuale carta nella tua mano vuoi giocare? ");
      scanf("%hu", &carta);
      --carta;
      if (carta < 0 || carta > 5) {
        printf("La mano è composta solo da 6 carte.\n");
      }
    } while (carta < 0 || carta > 5);
    Carta* Carta_Giocata = pPrimoMago -> Mano[carta];
    if (Carta_Giocata == NULL)
      printf("\nIn quello slot non c'è una carta.\n");
    else {
      switch (Carta_Giocata -> Tipo) {
        case 0: // Posiziona Creatura
          for (i = 0; i < 4; i++) { // Semplicemente se c'è uno slot libero piazzo la creatura
            if (pPrimoMago -> Campo[i] == NULL) {
              pPrimoMago -> Campo[i] = Carta_Giocata;
              printf("\nCreatura posizionata nello slot campo %d.\n", i+1);
              break;
            }
          }
          if (i == 4) {
            printf("\n\033[31mAttenzione!\033[0m Il campo è pieno, non puoi piazzare altre carte. La carta scelta verrà distrutta.\n");
          }
          break;

        case 1: // Usa Infliggi Danno
          printf("\nSu chi vuoi infliggere il danno?\n");
          printf("> 1: \033[96mMago avversario\033[0m\n");
          for (j = 0; j < 4; j++) { // Stampa la lista dei possibili bersagli
            Carta* Carta_Bersaglio = pSecondoMago -> Campo[j];
            if (Carta_Bersaglio != NULL) {
              ++k;
              printf("> %d: Slot campo %d - \033[96m%s %s\033[0m - Stats: \033[95m%d\033[0m\n", k+1, j+1, Carta_Bersaglio -> Nome, Carta_Bersaglio -> Aggettivo, Carta_Bersaglio -> Stats);
            }
          }
          do {
            printf("\033[92mScelta:\033[0m ");
            scanf("%hu", &Bersaglio);
            if (Bersaglio == 1) { // Se il bersaglio è il mago sottraggo i punti della creatura ai suoi
              pSecondoMago -> PV = pSecondoMago -> PV - Carta_Giocata -> Stats;
              printf("\nHai inflitto \033[95m%d\033[0m punti di danno a \033[96m%s\033[0m, i punti vita dell'avversario ora sono \033[95m%d\033[0m.\n", Carta_Giocata -> Stats, pSecondoMago -> Nickname, pSecondoMago -> PV);
            } else if (Bersaglio > 1 && Bersaglio <= k+1) { // Altrimenti attacco la creatura e controllo che sia stata sconfitta
              for (j = 0; j < 4; j++) {
                Carta* Carta_Bersaglio = pSecondoMago -> Campo[j];
                if (Carta_Bersaglio != NULL) {
                  ++l;
                  if (l == Bersaglio-1) {
                    Carta_Bersaglio -> Stats = Carta_Bersaglio -> Stats - Carta_Giocata -> Stats;
                    if (Carta_Bersaglio -> Stats < 1) {
                      printf("\nLa carta \033[96m%s %s\033[0m è stata sconfitta.\n", Carta_Bersaglio -> Nome, Carta_Bersaglio -> Aggettivo);
                      pSecondoMago -> Campo[j] = NULL;
                      free(pSecondoMago -> Campo[j]);
                    }
                    else {
                      printf("\nLa carta \033[96m%s %s\033[0m è stata danneggiata di \033[95m%d\033[0m punti, i suoi nuovi punti vita sono \033[95m%d\033[0m.\n", Carta_Bersaglio -> Nome, Carta_Bersaglio -> Aggettivo, Carta_Giocata -> Stats, Carta_Bersaglio -> Stats);
                    }
                  }
                }
              }
            } else {
              printf("\n\033[31mAttenzione!\033[0m Bersaglio non valido, inserisci un bersaglio tra quelli nella lista.\n");
            }
          } while (Bersaglio < 1 || Bersaglio > k+1);
          k = 0;
          break;

        case 2: // Usa Guarisci Danno
          printf("\nChi vuoi curare?\n");
          printf("> 1: \033[96mTu stesso\033[0m\n");
          for (j = 0; j < 4; j++) {
            Carta* Carta_Curata = pPrimoMago -> Campo[j];
            if (Carta_Curata != NULL) {
              ++k;
              printf("> %d: Slot campo %d - \033[96m%s %s\033[0m - Stats: \033[95m%d\033[0m\n", k+1, j+1, Carta_Curata -> Nome, Carta_Curata -> Aggettivo, Carta_Curata -> Stats);
            }
          }
          do {
            printf("\033[92mScelta:\033[0m ");
            scanf("%hu", &Bersaglio);
            if (Bersaglio == 1) {
              if (pPrimoMago -> PV == 20) { // Non puoi superare i 20 hp
                printf("\nNon puoi avere più di 20 Punti Vita. La carta giocata verrà distrutta.\n");
              } else if (pPrimoMago -> PV + Carta_Giocata -> Stats > 20) {
                pPrimoMago -> PV = 20;
                printf("\nHai recuperato tutti i Punti Vita, sei tornato a 20 Punti Vita.\n");
              } else {
                pPrimoMago -> PV = pPrimoMago -> PV + Carta_Giocata -> Stats;
                printf("\nI tuoi Punti Vita sono stati portati a \033[95m%d\033[0m.\n", pPrimoMago -> PV);
              }
            } else if (Bersaglio > 1 && Bersaglio <= k+1) {
              for (j = 0; j < 4; j++) {
                Carta* Carta_Curata = pPrimoMago -> Campo[j];
                if (Carta_Curata != NULL) {
                  ++l;
                  if (l == Bersaglio-1) {
                    Carta_Curata -> Stats = Carta_Curata -> Stats + Carta_Giocata -> Stats;
                    printf("\n\033[96m%s %s\033[0m si sente rinvigorito, ha ottenuto \033[95m%d\033[0m Punti Vita, portando il totale a \033[95m%d\033[0m.\n", Carta_Curata -> Nome, Carta_Curata -> Aggettivo, Carta_Giocata -> Stats, Carta_Curata -> Stats);
                  }
                }
              }
            } else {
              printf("\n\033[31mAttenzione!\033[0m Bersaglio non valido, inserisci un bersaglio tra quelli nella lista.\n");
            }
          } while (Bersaglio < 1 || Bersaglio > k+1);
          k = l = 0;
          break;

        case 3: // Usa Rimuovi Creatura
          for (i = 0; i < 4; i++) {
            if (pSecondoMago -> Campo[i] != NULL) {
              printf("\nQuale creatura vuoi rimuovere?\n");
              break;
            }
          }
          if (i == 4) {
            printf("\n\033[31mAttenzione!\033[0m Non ci sono creature da poter eliminare. La carta giocata verrà distrutta.\n");
          } else {
            for (i = 0; i < 4; i++) { // Stampo la lista dei possibili bersagli
              Carta* Carta_Bersaglio = pSecondoMago -> Campo[i];
              if (Carta_Bersaglio != NULL) {
                ++k;
                printf("> %d: Slot campo %d - \033[96m%s %s\033[0m - Stats: \033[95m%d\033[0m\n", k, i+1, Carta_Bersaglio -> Nome, Carta_Bersaglio -> Aggettivo, Carta_Bersaglio -> Stats);
              }
            }
            do {
              printf("\033[92mScelta:\033[0m ");
              scanf("%hu", &Bersaglio);
              for (j = 0; j < 4; j++) {
                Carta* Carta_Bersaglio = pSecondoMago -> Campo[j];
                if (Carta_Bersaglio != NULL) {
                  ++l;
                  if (l == Bersaglio) {
                    printf("\nLa creatura \033[96m%s %s\033[0m è stata distrutta.", Carta_Bersaglio -> Nome, Carta_Bersaglio -> Aggettivo);
                    pSecondoMago -> Campo[j] = NULL;
                    free(pSecondoMago -> Campo[j]);
                  }
                }
              }
            } while (Bersaglio < 1 || Bersaglio > l);
          }
          k = l = 0;
          break;
      }
    }
  } while(pPrimoMago -> Mano[carta] == NULL);
  pPrimoMago -> Mano[carta] = NULL;
  free(pPrimoMago -> Mano[carta]);
}

static void attacca(Mago* pPrimoMago, Mago* pSecondoMago) {
  unsigned short i = 0, j = 0, k = 0, l = 0, Bersaglio = 0, StatsTemp = 0, attacchi = 0;
  for (i = 0; i < 4; i++) {
    Carta* Attaccante = pPrimoMago -> Campo[i];
    if (pSecondoMago -> PV > 0) {
      if (Attaccante != NULL) { // Stampo la lista dei possibili bersagli
        attacchi = 1;
        printf("\nChi vuoi attaccare con \033[96m%s %s\033[0m?\n", Attaccante -> Nome, Attaccante -> Aggettivo);
        printf("> 1: \033[96mMago avversario\033[0m\n");
        k = l = 0;
        for (j = 0; j < 4; j++) {
          Carta* Carta_Bersaglio = pSecondoMago -> Campo[j];
          if (Carta_Bersaglio != NULL) {
            ++k;
            printf("> %d: Slot campo %d - \033[96m%s %s\033[0m - Stats: \033[95m%d\033[0m\n", k+1, j+1, Carta_Bersaglio -> Nome, Carta_Bersaglio -> Aggettivo, Carta_Bersaglio -> Stats);
          }
        }
        do {
          printf("\033[92mScelta:\033[0m ");
          scanf("%hu", &Bersaglio);
          if (Bersaglio == 1) { // Se il bersaglio è il mago sottraggo i punti della creatura ai suoi
            pSecondoMago -> PV = pSecondoMago -> PV - Attaccante -> Stats;
            printf("\nHai attaccato \033[96m%s\033[0m per \033[95m%d\033[0m punti, i punti vita dell'avversario ora sono \033[95m%d\033[0m.\n", pSecondoMago -> Nickname, Attaccante -> Stats, pSecondoMago -> PV);
          } else if (Bersaglio > 1 && Bersaglio <= k+1) { // Altrimenti attacco la creatura e controllo quale delle due esce vincitrice dallo scontro
            for (j = 0; j < 4; j++) {
              Carta* Carta_Bersaglio = pSecondoMago -> Campo[j];
              if (Carta_Bersaglio != NULL) {
                ++l;
                if (l == Bersaglio-1) {
                  if (Attaccante -> Stats == Carta_Bersaglio -> Stats) {
                    printf("\nLe due carte si sono distrutte a vicenda.\n");
                    pSecondoMago -> Campo[j] = NULL;
                    free(pSecondoMago -> Campo[j]);
                    pPrimoMago -> Campo[i] = NULL;
                    free(pPrimoMago -> Campo[i]);
                  } else {
                    StatsTemp = Carta_Bersaglio -> Stats;
                    pSecondoMago -> Campo[j] -> Stats = Carta_Bersaglio -> Stats - Attaccante -> Stats;
                    pPrimoMago -> Campo[i] -> Stats = Attaccante -> Stats - StatsTemp;
                    if (Carta_Bersaglio -> Stats < 1) {
                      printf("\nLa carta \033[96m%s %s\033[0m è stata sconfitta.\n", Carta_Bersaglio -> Nome, Carta_Bersaglio -> Aggettivo);
                      printf("\n\033[96m%s %s\033[0m alleato è rimasto con \033[95m%d\033[0m Punti Vita.\n", Attaccante -> Nome, Attaccante -> Aggettivo, Attaccante -> Stats);
                      pSecondoMago -> Campo[j] = NULL;
                      free(pSecondoMago -> Campo[j]);
                    } else if (Attaccante -> Stats < 1) {
                      printf("\n\033[96m%s %s\033[0m ha combattuto con tutte le forze, ma ne è uscito sconfitto.\n", Attaccante -> Nome, Attaccante -> Aggettivo);
                      printf("\n\033[96m%s %s\033[0m avversario è rimasto con \033[95m%d\033[0m Punti Vita.\n", Carta_Bersaglio -> Nome, Carta_Bersaglio -> Aggettivo, Carta_Bersaglio -> Stats);
                      pPrimoMago -> Campo[i] = NULL;
                      free(pPrimoMago -> Campo[i]);
                    }
                  }
                }
              }
            }
          } else {
            printf("\n\033[31mAttenzione!\033[0m Bersaglio non valido, inserisci un bersaglio tra quelli nella lista.\n");
          }
        } while (Bersaglio < 1 || Bersaglio > k+1);
      }
    }
  }
  if (!attacchi) {
    printf("\nNon hai creature con cui attaccare.\n");
  }
}
